--[[        ]]
--[[ Locals ]]
--[[        ]]

local proximity         = ProximityLib:GetInstance("1")
local RL                = AceLibrary("Roster-2.1")
local BC                = AceLibrary("Babble-Class-2.2")
local BS 				= AceLibrary("Babble-Spell-2.2")

local L                 = AceLibrary("AceLocale-2.2"):new("Litmus")


--[[                         ]]
--[[ RosterLib_RosterChanged ]]
--[[                         ]]

function Litmus:RosterLib_RosterUpdated(rosterTable)
	Litmus:CheckVisibility()
--~ 	if Litmus.db.profile.AutoGroupFilter and UnitInRaid("player") then
	if Litmus.db.profile.AutoGroupFilter then
		Litmus:AutoSubgroup()
	end
end

--[[              ]]
--[[ AutoSubgroup ]]
--[[              ]]

function Litmus:AutoSubgroup()
	
	if UnitInRaid("player") and Litmus.db.profile.AutoGroupFilter then
		
		local player = RL:GetUnitObjectFromUnit("player")
--~ 		Litmus:Print("subgroup player: "..player.subgroup)
				
--~ 		for i = 1, GetNumRaidMembers() do
--~ 			local unit = "raid"..i
--~ 			if UnitName(unit) == UnitName("player") then
--~ 				local RaidIDPlayer = unit
--~ 				i = GetNumRaidMembers()
--~ 			end
--~ 		end
--~ 		
--~ 		_, _, subgroup, _, _, _, _, _, _, _, _ = GetRaidRosterInfo(RaidIDPlayer)
		
		for i = 1, 8 do
			if i == player.subgroup then
				Litmus.db.profile.GroupFilter[i] = false
--~ 				Litmus:Print("group "..i.." = true")
			else
				Litmus.db.profile.GroupFilter[i] = true
--~ 				Litmus:Print("group "..i.." = false")
			end
		end
	end
end
	

--[[        ]]
--[[ Filter ]]
--[[        ]]

function Litmus:Filter(unit)
	if not UnitIsUnit("Player",unit.unitid) and (Litmus.db.profile.ClassFilter[unit.class]) then
		return true
	elseif not UnitIsUnit("Player",unit.unitid) and (GetNumRaidMembers() > 0 and Litmus.db.profile.GroupFilter[unit.subgroup]) then
		return true
	elseif not UnitIsUnit("Player",unit.unitid) and Litmus:UnitIsPet(unit) and Litmus.db.profile.PetFilter then
--~ 	elseif not UnitIsUnit("Player",unit.unitid) and Litmus.db.profile.PetFilter and not RAID_CLASS_COLORS[unit.unitid] then
		return true
	elseif UnitIsUnit("Player",unit.unitid) and Litmus.db.profile.SelfFilter then
		return true
	else
		return false
	end
end

--[[           ]]
--[[ UnitIsPet ]]
--[[           ]]

function Litmus:UnitIsPet(unit)
	if unit.unitid == "pet" then
		return true
	end
	
	for i = 1, 4 do
		if unit.unitid == "partypet"..i then
			return true
		end
	end
	
	for _, u in pairs(RL.roster) do
		if unit.unitid == "raidpet"..u.unitid then
			return true
		end
	end
	
	return false
	
--~ 	for _, u in pairs(RL.roster) do
--~ 		if unit.unitid == "pet" or "raidpet"..u.unitid or "partypet"..u.unitid then
--~ 			return true
--~ 		else
--~ 			return false
--~ 		end
--~ 	end
end


--[[            ]]
--[[ CheckUnits ]]
--[[            ]]

function Litmus:CheckUnits()
	
	-- Compost will keep us from GC the table between calls
	myTable = {}
	
	-- RL:Iterate() seems to be buggy, so we'll do it ourselves
	for _, u in pairs(RL.roster) do
		-- Check if unit has mana (to filter out druids in feral form)
		local PowerType = UnitPowerType(u.unitid)
		-- Check if we should filter the unit out
		if not Litmus:Filter(u) and (PowerType == 0) then
			
			-- Get the current mana percentage of this unit
			local pct = (UnitMana(u.unitid) / UnitManaMax(u.unitid)) * 100
			
			-- If the unit has lost mana and is still alive, we'll attempt to place it in our list
			if pct <= Litmus.db.profile.ScanMana and not UnitIsDeadOrGhost(u.unitid) then self:CheckToShow(myTable,u,1)
			end
		end
		
	end
	
	-- Reset our display frames to show the current list of damaged units
	self:DisplayUnits(myTable)
end

--[[             ]]
--[[ CheckToShow ]]
--[[             ]]

function Litmus:CheckToShow(tbl,unit,i)
   -- We shouldn't ever be below 1, but check for it.  We will be above the max
   -- when we've pushed a previous value out, and they no longer fit in the list
   if i > Litmus.db.profile.ScanUnits or i < 1 then return end

   -- Get health percent for the current unit
   local currPct  = UnitMana(unit.unitid) / UnitManaMax(unit.unitid)
   
   -- If we're checking a fresh unit, make sure that they belong in our list
   -- before checking for the position to place it
   if i == 1 and not tbl[Litmus.db.profile.ScanUnits] == nil then
      local checkPct = UnitMana(tbl[Litmus.db.profile.ScanUnits].unitid) /
         UnitManaMax(tbl[Litmus.db.profile.ScanUnits].unitid)
      if currPct > checkPct then return end
   end
   
   -- If there's nothing at this table entry, place the unit here and finish
   if tbl[i] == nil then
      tbl[i] = unit
      return
   end
   
   -- Otherwise, check our units health against the unit at the ith position
   -- in our table
   local checkPct = UnitMana(tbl[i].unitid) / UnitManaMax(tbl[i].unitid)

   -- If we're not as damaged as this unit, check against the next
   if currPct >= checkPct then
      self:CheckToShow(tbl,unit,i+1)
      
   -- We're more damage than this unit.  Replace it, see if there's still a
   -- place for this unit somewhere else in our table
   else
      local prevUnit = tbl[i]
      tbl[i] = unit
      self:CheckToShow(tbl,prevUnit,i+1)
   end
end

--[[              ]]
--[[ DisplayUnits ]]
--[[              ]]

function Litmus:DisplayUnits(tbl)
	
	local pct = nil
	-- Loop through all the unit frame bars we've created since being enabled
	-- (we won't have more damaged units then there are bars)
	for i in pairs(Litmus.frames.units) do
		
		-- If there's a unit to display for this frame, simply update that
		-- frames data, make sure it's shown.
		if tbl[i] then
			Litmus.frames.units[i].unit = tbl[i]
			pct = (UnitMana(tbl[i].unitid) / UnitManaMax(tbl[i].unitid)) * 100
			
			Litmus.frames.units[i].Name:SetText(tbl[i].name)
			Litmus.frames.units[i].Name:SetTextColor(self:GetNameColor(tbl[i].class))
			Litmus.frames.units[i].BarBG:SetStatusBarColor( Litmus:FindBarColor(true,tbl[i]) )
			Litmus.frames.units[i].Bar:SetStatusBarColor( Litmus:FindBarColor(false,tbl[i]) )
			Litmus.frames.units[i].Bar:SetValue(pct)
			Litmus.frames.units[i].Missing:SetText(" -" .. UnitManaMax(tbl[i].unitid) - UnitMana(tbl[i].unitid))
			Litmus.frames.units[i].Missing:SetTextColor(self.db.profile.TextColor.r, self.db.profile.TextColor.g, self.db.profile.TextColor.b)
			Litmus.frames.units[i].GroupNumber:SetText(tbl[i].subgroup)
			Litmus.frames.units[i].GroupNumber:SetTextColor(self.db.profile.TextColor.r, self.db.profile.TextColor.g, self.db.profile.TextColor.b)
			
			if Litmus:UnitHasInnervate(tbl[i].unitid) then
				Litmus.frames.units[i].Innervate.texture:SetTexture("Interface\\Icons\\Spell_Nature_Lightning")
			else
				Litmus.frames.units[i].Innervate.texture:SetTexture(0,0,0,0.15)
			end
			Litmus.frames.units[i]:Show()
			
		-- Otherwise, make sure any extra bars are hidden.  I deliberately don't
		-- hide all the frame bars, and then only show the necessary ones, as the
		-- frame bar will seem unresponsive to mouse clicks (the click is actually
		-- occurring between the hide/show) despite the bar always being visible
		else
			Litmus.frames.units[i]:Hide()
		end
		
	end
	
end

--[[                  ]]
--[[ UnitHasInnervate ]]
--[[                  ]]

function Litmus:UnitHasInnervate(unit)
	
	local i = 1
	local name
	
	repeat
		name, _, _, _ = UnitBuff(unit, i)
		
		if name == BS["Innervate"] then
			return true
		end
		
		i = i+1
	until not name
	
	return nil
end

